{ ncurses.pas   for GNU Pascal }

{
    Unit to access the ncurses library

    Derived from curses.h and c source by Eric S. Raymond, Zeyd M. Ben-Halim and others.

    FPC version: (c) 1998 by Michael Van Canneyt, member Free Pascal development 
       team, who granted permission to include his mods with GPC distribution 
       (email, 14 March 2005).  from: $Id: ncurses.pp,v 1.9 2005/02/14 
       Many thanks to Ken Wright for his patches. 

    Mods for GNU Pascal (GPC):
      Initial GPC port by Prof. Abimbola A Olowofoyeku 15 March 2005
      Subsequent GPC mods by Willett Kempton, April 2005:
       - removed "cdecl;" and "libncurses", needed only for FPC
       - change integer to ShortInt
       - change bool to ByteBool, one MedBool, so Pascal caller can use true, false
       - add Curses_True, Curses_False, Curses_ERR, Curses_OK, like Ada ncurses
       - code had too many ifdefs for readability; much but not all non-GPC Pascal pulled

    Remaining problems with GPC version of ncurses
       - many int32 apparently should be ShortInt (int16), not fully investigated
       - note: setterm is depreciated and here is disabled

      GPC compile:   gpc -lncurses ncurses.pas
      this version tested on gpc 20030830, based on gcc-3.3.2.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

 **********************************************************************}

UNIT ncurses;

INTERFACE

{$ifdef __GPC__}
   {$define exit Return}
   {$X+}
   {$L ncurses}
   {.$W-}
{$else} { GPC }
{$packrecords C}
{$ifdef OpenBSD}                // openbsd curses = ncurses. Openbsd ocurses = old curses.
{$linklib curses}
{$else}
{$linklib ncurses}
{$endif}
{$linklib c}
{$endif} { GPC }

{$ifdef __GPC__}
TYPE
  int32 = integer attribute ( size = 32 ); 
{$else}
TYPE
  int32 = longint;
{$endif}

TYPE
  bool = ByteBool;  { was byte, values 0,1. ByteBool allows&requires caller to use Pascal false,true }
  IntBool = MedBool; { was int in c (signed 32 bit integer), used only as a boolean }
  boolERR = ByteInt; { c bool type, but used for TRUE, FALSE and ERR }
  PINTEGER = ^int32;
  PLongint = ^ int32;
  PFILE = pointer;

CONST
  NCURSES_VERSION_MAJOR = 5;
  NCURSES_VERSION_MINOR = 0;
  NCURSES_VERSION_PATCH = 19991023;
  NCURSES_VERSION = '5.0';

TYPE
  chtype  = int32;  { type returned from getch, etc; equivalent type in GCC is int, in GPC is integer }
  pchtype = pchar;

CONST
  CXX_BUILTIN_BOOL = 1;   { not used? }
TYPE
  CXX_TYPE_OF_BOOL = ByteBool;  { was char; }

{ the Ada ncurses interface defines constants to avoid conflicts with Ada }
{   built-in true/false.  Used as  "if somefunctionreturn = Curses_True then ..."  }
CONST 
   Curses_True = 1;   Curses_False = 0;

VAR
    COLORS : int32; external  name 'COLORS';
    COLOR_PAIRS : int32; external  name 'COLOR_PAIRS';

    CONST
       COLOR_BLACK = 0;
       COLOR_RED = 1;
       COLOR_GREEN = 2;
       COLOR_YELLOW = 3;
       COLOR_BLUE = 4;
       COLOR_MAGENTA = 5;
       COLOR_CYAN = 6;
       COLOR_WHITE = 7;

TYPE
    tacs_map = ARRAY [char] OF chtype; { 128-byte, re-settable array, initially char->chtype }
    pacs_map = ^tacs_map;

VAR
    acs_map : tacs_map; external name 'acs_map';  { some platforms use  '_nc_acs_map'? }

    FUNCTION ACS_ULCORNER : chtype;
    FUNCTION ACS_LLCORNER : chtype;
    FUNCTION ACS_URCORNER : chtype;
    FUNCTION ACS_LRCORNER : chtype;
    FUNCTION ACS_LTEE     : chtype;
    FUNCTION ACS_RTEE     : chtype;
    FUNCTION ACS_BTEE     : chtype;
    FUNCTION ACS_TTEE     : chtype;
    FUNCTION ACS_HLINE    : chtype;
    FUNCTION ACS_VLINE    : chtype;
    FUNCTION ACS_PLUS     : chtype;
    FUNCTION ACS_S1       : chtype;
    FUNCTION ACS_S9       : chtype;
    FUNCTION ACS_DIAMOND  : chtype;
    FUNCTION ACS_CKBOARD  : chtype;
    FUNCTION ACS_DEGREE   : chtype;
    FUNCTION ACS_PLMINUS  : chtype;
    FUNCTION ACS_BULLET   : chtype;
    FUNCTION ACS_LARROW   : chtype;
    FUNCTION ACS_RARROW   : chtype;
    FUNCTION ACS_DARROW   : chtype;
    FUNCTION ACS_UARROW   : chtype;
    FUNCTION ACS_BOARD    : chtype;
    FUNCTION ACS_LANTERN  : chtype;
    FUNCTION ACS_BLOCK    : chtype;
    FUNCTION ACS_S3       : chtype;
    FUNCTION ACS_S7       : chtype;
    FUNCTION ACS_LEQUAL   : chtype;
    FUNCTION ACS_GEQUAL   : chtype;
    FUNCTION ACS_PI       : chtype;
    FUNCTION ACS_NEQUAL   : chtype;
    FUNCTION ACS_STERLING : chtype;
    {
       Line drawing ACS names are of the form ACS_trbl, where t is the top, r
       is the right, b is the bottom, and l is the left.  t, r, b, and l might
       be B (blank), S (single), D (double), or T (thick).  The subset defined
       here only uses B and S.
     }
    {
    #define ACS_BSSB    ACS_ULCORNER
    #define ACS_SSBB    ACS_LLCORNER
    #define ACS_BBSS    ACS_URCORNER
    #define ACS_SBBS    ACS_LRCORNER
    #define ACS_SBSS    ACS_RTEE
    #define ACS_SSSB    ACS_LTEE
    #define ACS_SSBS    ACS_BTEE
    #define ACS_BSSS    ACS_TTEE
    #define ACS_BSBS    ACS_HLINE
    #define ACS_SBSB    ACS_VLINE
    #define ACS_SSSS    ACS_PLUS
    }

    CONST
       ERR = - ( 1 );
       OK = 0;
       { Ada-like variants of ERR/OK (unlike true/false, not necessary): }
       Curses_ERR = ERR;
       Curses_OK  = OK;
       _SUBWIN = $01;
       _ENDLINE = $02;
       _FULLWIN = $04;
       _SCROLLWIN = $08;
       _ISPAD = $10;
       _HASMOVED = $20;
       _WRAPPED = $40;
    {
       this value is used in the firstchar and lastchar fields to mark
       unchanged lines
     }
       _NOCHANGE = - ( 1 );
    {
       this value is used in the oldindex field to mark lines created by insertions
       and scrolls.
     }
       _NEWINDEX = - ( 1 );
    {
    typedef struct screen  SCREEN;
    typedef struct _win_st WINDOW;
    }

    TYPE

       attr_t = chtype;
       ldat = RECORD
            text : ^chtype;
            firstchar : ShortInt;
            lastchar : ShortInt;
            oldindex : ShortInt;
         END;

       _win_st = RECORD
            _cury : ShortInt;
            _curx : ShortInt;
            _maxy : ShortInt;
            _maxx : ShortInt;
            _begy : ShortInt;
            _begx : ShortInt;
            _flags : ShortInt;
            _attrs : attr_t;
            _bkgd : chtype;
            _notimeout : bool;
            _clear : bool;
            _leaveok : bool;
            _scroll : bool;
            _idlok : bool;
            _idcok : bool;
            _immed : bool;
            _sync : bool;
            _use_keypad : bool;
            _delay : int32;
            _line : ^ldat;
            _regtop : ShortInt;
            _regbottom : ShortInt;
            _parx : int32;
            _pary : int32;
            _parent : ^WINDOW;
            _pad : RECORD
                 _pad_y : ShortInt;
                 _pad_x : ShortInt;
                 _pad_top : ShortInt;
                 _pad_left : ShortInt;
                 _pad_bottom : ShortInt;
                 _pad_right : ShortInt;
              END;
            _yoffset : ShortInt;
         END;
        WINDOW = _win_st;
        PWINDOW = ^WINDOW;
       SCREEN = WINDOW;
       PSCREEN = PWINDOW;

      VAR
       stdscr  : PWINDOW; external name 'stdscr';
       curscr  : PWINDOW; external name 'curscr';
       newscr  : PWINDOW; external name 'newscr';
       LINES   : int32; external name 'LINES';
       COLS    : int32; external name 'COLS';
       TABSIZE : int32; external name 'TABSIZE';
       ESCDELAY : int32; external name 'ESCDELAY';

    FUNCTION define_key ( _para1 : pchar; _para2 : int32 ) : int32; external name 'define_key';
    FUNCTION keyok ( _para1 : int32; _para2 : bool ) : int32; external name 'keyok';
    FUNCTION resizeterm ( _para1 : int32; _para2 : int32 ) : int32; external name 'resizeterm';
    FUNCTION use_default_colors : int32; external name 'use_default_colors';
    FUNCTION wresize ( _para1 : pWINDOW; _para2 : int32; _para3 : int32 ) : int32; external name 'wresize';
    {
    extern char ttytype[];
    }
    FUNCTION baudrate : int32; external name 'baudrate';
    FUNCTION beep : int32; external name 'beep';
    FUNCTION can_change_color : bool; external name 'can_change_color';
    FUNCTION cbreak : int32; external name 'cbreak';
    FUNCTION clearok ( _para1 : pWINDOW; _para2 : bool ) : int32; external name 'clearok';
    FUNCTION color_content ( _para1 : int32; _para2 : plongint; _para3 : plongint; _para4 : plongint ) : int32; external name 'color_content';

    FUNCTION copywin ( _para1 : pWINDOW; _para2 : pWINDOW; _para3 : int32; _para4 : int32; _para5 : int32;
               _para6 : int32; _para7 : int32; _para8 : int32; _para9 : int32 ) : int32;external name 'copywin';
    FUNCTION curs_set ( _para1 : int32 ) : int32; external name 'curs_set';
    FUNCTION def_prog_mode : int32; external name 'def_prog_mode';
    FUNCTION def_shell_mode : int32; external name 'def_shell_mode';
    FUNCTION delay_output ( _para1 : int32 ) : int32; external name 'delay_output';
    PROCEDURE delscreen ( _para1 : pSCREEN );external name 'delscreen';
    FUNCTION delwin ( _para1 : pWINDOW ) : int32; external name 'delwin';

    FUNCTION doupdate : int32; external name 'doupdate';

    FUNCTION echo : int32; external name 'echo';
    FUNCTION endwin : int32; external name 'endwin';
    FUNCTION erasechar : char; external name 'erasechar';
    PROCEDURE filter;external name 'filter';
    FUNCTION flash : int32; external name 'flash';
    FUNCTION flushinp : int32; external name 'flushinp';

    FUNCTION halfdelay ( _para1 : int32 ) : int32; external name 'halfdelay';
    FUNCTION has_colors : bool; external name 'has_colors';
    FUNCTION has_ic : bool; external name 'has_ic';
    FUNCTION has_il : bool; external name 'has_il';
    PROCEDURE idcok ( _para1 : pWINDOW; _para2 : bool );external name 'idcok';
    FUNCTION idlok ( _para1 : pWINDOW; _para2 : bool ) : int32; external name 'idlok';
    PROCEDURE immedok ( _para1 : pWINDOW; _para2 : bool );external name 'immedok';

    FUNCTION init_color ( _para1 : int32; _para2 : int32; _para3 : int32; _para4 : int32 ) : int32; external name 'init_color';
    FUNCTION init_pair ( _para1 : int32; _para2 : int32; _para3 : int32 ) : int32; external name 'init_pair';
    FUNCTION intrflush ( _para1 : pWINDOW; _para2 : bool ) : int32; external name 'intrflush';
    FUNCTION isendwin : bool; external name 'isendwin';
    FUNCTION is_linetouched ( _para1 : pWINDOW; _para2 : int32 ) : BoolERR; external name 'is_linetouched';
    FUNCTION is_wintouched ( _para1 : pWINDOW ) : BoolERR; external name 'is_wintouched';


    FUNCTION keypad ( _para1 : pWINDOW; _para2 : bool ) : int32; external name 'keypad';
    FUNCTION killchar : char; external name 'killchar';
    FUNCTION leaveok ( _para1 : pWINDOW; _para2 : bool ) : int32; external name 'leaveok';

    FUNCTION meta ( _para1 : pWINDOW; _para2 : bool ) : int32; external name 'meta';
    FUNCTION mvcur ( _para1 : int32; _para2 : int32; _para3 : int32; _para4 : int32 ) : int32; external name 'mvcur';
    FUNCTION mvderwin ( _para1 : pWINDOW; _para2 : int32; _para3 : int32 ) : int32; external name 'mvderwin';
    FUNCTION mvprintw ( _para1 : int32;_para2 : int32;_para3 : pchar;{$ifdef __GPC__}...{$else}_para4 : ARRAY OF CONST{$endif} ) : int32; external name 'mvprintw';
    {
    extern int mvscanw(int,int,const char  ,...)
                GCC_SCANFLIKE(3,4);
    }
    FUNCTION mvwin ( _para1 : pWINDOW; _para2 : int32; _para3 : int32 ) : int32; external name 'mvwin';
    FUNCTION mvwprintw ( _para1 : pWINDOW;_para2, _para3 : int32;_para4 : pchar;{$ifdef __GPC__}...{$else}_para5 : ARRAY OF CONST{$endif} ) : int32; external name 'mvwprintw';
    {
    extern int mvwprintw(WINDOW ,int,int,const char  ,...)
                GCC_PRINTFLIKE(4,5);
    extern int mvwscanw(WINDOW  ,int,int,const char  ,...)
                GCC_SCANFLIKE(4,5);
    }
    FUNCTION napms ( _para1 : int32 ) : int32; external name 'napms';

    FUNCTION nl : int32; external name 'nl';
    FUNCTION nocbreak : int32; external name 'nocbreak';
    FUNCTION nodelay ( _para1 : pWINDOW; _para2 : bool ) : int32; external name 'nodelay';
    FUNCTION noecho : int32; external name 'noecho';
    FUNCTION nonl : int32; external name 'nonl';
    FUNCTION noqiflush : int32; external name 'noqiflush';
    FUNCTION noraw : int32; external name 'noraw';
    FUNCTION notimeout ( _para1 : pWINDOW; _para2 : bool ) : int32; external name 'notimeout';

    FUNCTION overlay ( _para1 : pWINDOW; _para2 : pWINDOW ) : int32; external name 'overlay';

    FUNCTION overwrite ( _para1 : pWINDOW; _para2 : pWINDOW ) : int32; external name 'overwrite';
    FUNCTION pair_content ( _para1 : int32; _para2 : plongint; _para3 : plongint ) : int32; external name 'pair_content';

    FUNCTION pechochar ( _para1 : pWINDOW; _para2 : chtype ) : int32; external name 'pechochar';
    FUNCTION pnoutrefresh ( _para1 : pWINDOW; _para2 : int32; _para3 : int32; _para4 : int32; _para5 : int32;
               _para6 : int32; _para7 : int32 ) : int32;external name 'pnoutrefresh';
    FUNCTION prefresh ( _para1 : pWINDOW; _para2 : int32; _para3 : int32; _para4 : int32; _para5 : int32;
               _para6 : int32; _para7 : int32 ) : int32;external name 'prefresh';
    {
    extern int printw(const char  ,...)
                GCC_PRINTFLIKE(1,2);
    }
    FUNCTION putp ( _para1 : pchar ) : int32; external name 'putp';
    FUNCTION putwin ( _para1 : pWINDOW; _para2 : pFILE ) : int32; external name 'putwin';
    FUNCTION qiflush : int32; external name 'qiflush';
    FUNCTION raw : int32; external name 'raw';
    FUNCTION resetty : int32; external name 'resetty';
    FUNCTION reset_prog_mode : int32; external name 'reset_prog_mode';
    FUNCTION reset_shell_mode : int32; external name 'reset_shell_mode';
(*
    function ripoffline(_para1:int32; init:function (_para1:pWINDOW; _para2:int32):int32):int32; external name 'ripoffline';
*)
    FUNCTION savetty : int32; external name 'savetty';
    {
    extern int scanw(const char  ,...)
                GCC_SCANFLIKE(1,2);
    }
    FUNCTION scr_dump ( _para1 : pchar ) : int32; external name 'scr_dump';

    FUNCTION scr_init ( _para1 : pchar ) : int32; external name 'scr_init';
    FUNCTION scrollok ( _para1 : pWINDOW; _para2 : bool ) : int32; external name 'scrollok';

    FUNCTION scr_restore ( _para1 : pchar ) : int32; external name 'scr_restore';
    FUNCTION scr_set ( _para1 : pchar ) : int32; external name 'scr_set';

    FUNCTION slk_attroff ( _para1 : attr_t ) : int32; external name 'slk_attroff';
    FUNCTION slk_attron ( _para1 : attr_t ) : int32; external name 'slk_attron';
    FUNCTION slk_attrset ( _para1 : attr_t ) : int32; external name 'slk_attrset';
    FUNCTION slk_attr : attr_t; external name 'slk_attr';
    FUNCTION slk_clear : int32; external name 'slk_clear';
    FUNCTION slk_init ( _para1 : int32 ) : int32; external name 'slk_init';

    FUNCTION slk_noutrefresh : int32; external name 'slk_noutrefresh';
    FUNCTION slk_refresh : int32; external name 'slk_refresh';
    FUNCTION slk_restore : int32; external name 'slk_restore';

    FUNCTION slk_set ( _para1 : int32; _para2 : pchar; _para3 : int32 ) : int32; external name 'slk_set';
    FUNCTION slk_touch : int32; external name 'slk_touch';
    FUNCTION start_color : int32; external name 'start_color';

    FUNCTION syncok ( _para1 : pWINDOW; _para2 : bool ) : int32; external name 'syncok';
    FUNCTION termattrs : chtype; external name 'termattrs';

    FUNCTION tigetflag ( _para1 : pchar ) : int32; external name 'tigetflag';

    FUNCTION tigetnum ( _para1 : pchar ) : int32; external name 'tigetnum';

    FUNCTION derwin ( _para1 : pWINDOW; _para2 : int32; _para3 : int32; _para4 : int32; _para5 : int32 ) : PWINDOW; external name 'derwin';
    FUNCTION dupwin ( _para1 : pWINDOW ) : PWINDOW; external name 'dupwin';
    FUNCTION getwin ( _para1 : pFILE ) : PWINDOW; external name 'getwin';
    FUNCTION initscr : PWINDOW; external name 'initscr';
    FUNCTION keyname  ( _para1 : int32 ) : pchar; external name 'keyname';
    FUNCTION longname : pchar; external name 'longname';
    FUNCTION newpad ( _para1 : int32; _para2 : int32 ) : PWINDOW; external name 'newpad';
    FUNCTION newterm ( _para1 : pchar; _para2 : pFILE; _para3 : pFILE ) : PSCREEN; external name 'newterm';
    FUNCTION newwin  ( _para1 : int32; _para2 : int32; _para3 : int32; _para4 : int32 ) : PWINDOW; external name 'newwin';
    FUNCTION set_term ( _para1 : pSCREEN ) : PSCREEN; external name 'set_term';
    FUNCTION slk_label ( _para1 : int32 ) : pchar; external name 'slk_label';
    FUNCTION subpad ( _para1 : pWINDOW; _para2 : int32; _para3 : int32; _para4 : int32; _para5 : int32 ) : PWINDOW; external name 'subpad';
    FUNCTION subwin ( _para1 : pWINDOW; _para2 : int32; _para3 : int32; _para4 : int32; _para5 : int32 ) : PWINDOW; external name 'subwin';
    FUNCTION termname : pchar; external name 'termname';
    FUNCTION tigetstr ( _para1 : pchar ) : pchar; external name 'tigetstr';
    FUNCTION typeahead ( _para1 : int32 ) : int32; external name 'typeahead';
    FUNCTION ungetch ( _para1 : int32 ) : int32; external name 'ungetch';
    PROCEDURE use_env ( _para1 : bool );external name 'use_env';
    FUNCTION vidattr ( _para1 : chtype ) : int32; external name 'vidattr';
(*
{
    function vidputs(_para1:chtype; _para2:function (_para1:int32):int32):int32; external name 'vidputs';
}
{
    function vwprintw(_para1:pWINDOW; _para2:pchar; _para3:va_list):int32; external name 'vwprintw';
    function vwscanw(_para1:pWINDOW; _para2:pchar; _para3:va_list):int32; external name 'vwscanw';
}
*)
    FUNCTION waddch ( _para1 : pWINDOW; _para2 : chtype ) : int32; external name 'waddch';
    FUNCTION waddchnstr ( _para1 : pWINDOW; _para2 : pchtype; _para3 : int32 ) : int32; external name 'waddchnstr';
    FUNCTION waddnstr ( _para1 : pWINDOW; _para2 : pchar; _para3 : int32 ) : int32; external name 'waddnstr';
    FUNCTION wattr_on ( _para1 : pWINDOW; _para2 : attr_t ) : int32; external name 'wattr_on';
    FUNCTION wattr_off ( _para1 : pWINDOW; _para2 : attr_t ) : int32; external name 'wattr_off';
    FUNCTION wattr_set ( win : pwindow; at : int32 ) : int32; external name 'wattr_set';
    FUNCTION wattron ( win : pwindow;at : int32 ) : int32; external name 'wattron';
    FUNCTION wattroff ( win : pwindow;at : int32 ) : int32; external name 'wattroff';
    FUNCTION wattrset ( win : pwindow;at : int32 ) : int32; external name 'wattrset';
    FUNCTION wbkgd ( _para1 : pWINDOW; _para2 : chtype ) : int32; external name 'wbkgd';
    PROCEDURE wbkgdset ( _para1 : pWINDOW; _para2 : chtype );external name 'wbkgdset';
    FUNCTION wborder ( _para1 : pWINDOW; _para2 : chtype; _para3 : chtype; _para4 : chtype; _para5 : chtype;
               _para6 : chtype; _para7 : chtype; _para8 : chtype; _para9 : chtype ) : int32;external name 'wborder';
    FUNCTION wchgat ( _para1 : pWINDOW; _para2 : int32; _para3 : attr_t; _para4 : int32; _para5 : pointer ) : int32; external name 'wchgat';
    FUNCTION wclear ( _para1 : pWINDOW ) : int32; external name 'wclear';
    FUNCTION wclrtobot ( _para1 : pWINDOW ) : int32; external name 'wclrtobot';
    FUNCTION wclrtoeol ( _para1 : pWINDOW ) : int32; external name 'wclrtoeol';
    PROCEDURE wcursyncup ( _para1 : pWINDOW );external name 'wcursyncup';
    FUNCTION wdelch ( _para1 : pWINDOW ) : int32; external name 'wdelch';
    FUNCTION wechochar ( _para1 : pWINDOW; _para2 : chtype ) : int32; external name 'wechochar';
    FUNCTION werase ( _para1 : pWINDOW ) : int32; external name 'werase';
    FUNCTION wgetch ( _para1 : pWINDOW ) : int32; external name 'wgetch';
    FUNCTION wgetnstr ( _para1 : pWINDOW; _para2 : pchar; _para3 : int32 ) : int32; external name 'wgetnstr';
    FUNCTION whline ( _para1 : pWINDOW; _para2 : chtype; _para3 : int32 ) : int32; external name 'whline';
    FUNCTION winch ( win : PWindow ) : int32; external name 'winch';
    FUNCTION winchnstr ( _para1 : pWINDOW; _para2 : pchtype; _para3 : int32 ) : int32; external name 'winchnstr';
    FUNCTION winnstr ( _para1 : pWINDOW; _para2 : pchar; _para3 : int32 ) : int32; external name 'winnstr';
    FUNCTION winsch ( _para1 : pWINDOW; _para2 : chtype ) : int32; external name 'winsch';
    FUNCTION winsdelln ( _para1 : pWINDOW; _para2 : int32 ) : int32; external name 'winsdelln';
    FUNCTION winsnstr ( _para1 : pWINDOW; _para2 : pchar; _para3 : int32 ) : int32; external name 'winsnstr';
    FUNCTION wmove ( _para1 : pWINDOW; _para2 : int32; _para3 : int32 ) : int32; external name 'wmove';
    FUNCTION wnoutrefresh ( _para1 : pWINDOW ) : int32; external name 'wnoutrefresh';
    {
    extern int wprintw(WINDOW  ,const char  ,...)
                GCC_PRINTFLIKE(2,3);
    }
    FUNCTION wredrawln ( _para1 : pWINDOW; _para2 : int32; _para3 : int32 ) : int32; external name 'wredrawln';
    FUNCTION wrefresh ( _para1 : pWINDOW ) : int32; external name 'wrefresh';
    {
    extern int wscanw(WINDOW  ,const char  ,...)
                GCC_SCANFLIKE(2,3);
    }
    FUNCTION wscrl ( _para1 : pWINDOW; _para2 : int32 ) : int32; external name 'wscrl';
    FUNCTION wsetscrreg ( _para1 : pWINDOW; _para2 : int32; _para3 : int32 ) : int32; external name 'wsetscrreg';
    PROCEDURE wsyncdown ( _para1 : pWINDOW );external name 'wsyncdown';
    PROCEDURE wsyncup ( _para1 : pWINDOW );external name 'wsyncup';
    FUNCTION wtimeout ( _para1 : pWINDOW; _para2 : int32 ) : int32; external name 'wtimeout';
    FUNCTION wtouchln ( _para1 : pWINDOW; _para2 : int32; _para3 : int32; _para4 : int32 ) : int32; external name 'wtouchln';
    FUNCTION wvline ( _para1 : pWINDOW; _para2 : chtype; _para3 : int32 ) : int32; external name 'wvline';
    FUNCTION mvwchgat ( _para1 : pWINDOW; _para2 : int32; _para3 : int32;
                      _para4 : int32; _para5 : int32; _para6 : int32;
                      _para7 : int32 ) : int32;external name 'mvwchgat';
    FUNCTION PAIR_NUMBER ( _para1 : int32 ) : int32;external name 'PAIR_NUMBER';

    CONST
           A_NORMAL = 0;
           A_ATTRIBUTES = ( NOT 0 ) SHL 8;
           A_CHARTEXT = ( 1 SHL ( 0 + 8 ) ) - 1;
           A_COLOR = ( ( 1 SHL 8 ) - 1 ) SHL 8;
           A_STANDOUT = 1 SHL ( 8 + 8 );
           A_UNDERLINE = 1 SHL ( 9 + 8 );
           A_REVERSE = 1 SHL ( 10 + 8 );
           A_BLINK = 1 SHL ( 11 + 8 );
           A_DIM = 1 SHL ( 12 + 8 );
           A_BOLD = 1 SHL ( 13 + 8 );
           A_ALTCHARSET = 1 SHL ( 14 + 8 );
           A_INVIS = 1 SHL ( 15 + 8 );
           A_PROTECT = 1 SHL ( 16 + 8 );
           A_HORIZONTAL = 1 SHL ( 17 + 8 );
           A_LEFT = 1 SHL ( 18 + 8 );
           A_LOW = 1 SHL ( 19 + 8 );
           A_RIGHT = 1 SHL ( 20 + 8 );
           A_TOP = 1 SHL ( 21 + 8 );
           A_VERTICAL = 1 SHL ( 22 + 8 );
     FUNCTION color_pair ( n : int32 ) : int32;
{
           PAIR_NUMBER = (a(@(A_COLOR))) shr 8;
}

    {
       pseudo functions
     }
    FUNCTION wgetstr ( w : pwindow;s : pchar ) : int32;
    FUNCTION getnstr ( s : pchar;n : int32 ) : int32;
    FUNCTION setterm ( term : int32 ) : int32;
    FUNCTION fixterm : int32;
    FUNCTION resetterm : int32;
    FUNCTION saveterm : int32;
    FUNCTION crmode : int32;
    FUNCTION nocrmode : int32;
    PROCEDURE getyx   ( win : pwindow; VAR y, x : int32 );
    PROCEDURE getbegyx ( win : pwindow; VAR y, x : int32 );
    PROCEDURE getmaxyx ( win : pwindow; VAR y, x : int32 );
    PROCEDURE getparyx ( win : pwindow; VAR y, x : int32 );
    PROCEDURE getsyx  ( VAR y, x : int32 );
    PROCEDURE setsyx ( y, x : int32 );
    FUNCTION getattrs ( win : pwindow ) : int32;
    FUNCTION getcurx ( win : pwindow ) : int32;
    FUNCTION getcury ( win : pwindow ) : int32;
    FUNCTION getbegx ( win : pwindow ) : int32;
    FUNCTION getbegy ( win : pwindow ) : int32;
    FUNCTION getmaxx ( win : pwindow ) : int32;
    FUNCTION getmaxy ( win : pwindow ) : int32;
    FUNCTION getparx ( win : pwindow ) : int32;
    FUNCTION getpary ( win : pwindow ) : int32;
    FUNCTION wstandout ( win : pwindow ) : int32;
    FUNCTION wstandend ( win : pwindow ) : int32;
{kjw, 08/24/2000, changed to  external
    function wattr_set(win : pwindow; at : int32) : int32;
    function wattron(win : pwindow;at : int32) : int32;
    function wattroff(win : pwindow;at : int32) : int32;
    function wattrset(win : pwindow;at : int32) : int32;
}
    FUNCTION scroll ( win : pwindow ) : int32;
    FUNCTION touchwin ( win : pwindow ) : int32;
    FUNCTION touchline ( win : pwindow;s, c : int32 ) : int32;
    FUNCTION untouchwin ( win : pwindow ) : int32;
    FUNCTION box ( win : pwindow;v, h : int32 ) : int32;
    FUNCTION border ( ls, rs, ts, bs, tl, tr, bl, br : int32 ) : int32;
    FUNCTION hline ( ch, n : int32 ) : int32;
    FUNCTION vline ( ch, n : int32 ) : int32;
    FUNCTION winstr ( w : pwindow;s : pchar ) : int32;
    FUNCTION winchstr ( w : pwindow;s : pchar ) : int32;
    FUNCTION winsstr ( w : pwindow;s : pchar ) : int32;
    FUNCTION redrawwin ( w : pwindow ) : int32;
    FUNCTION waddstr ( win : pwindow;st : pchar ) : int32;
    FUNCTION waddchstr ( win : pwindow;st : pchar ) : int32;
    {
       pseudo functions for standard screen
     }
    FUNCTION addch ( ch : int32 ) : int32;
    FUNCTION addchnstr ( st : pchar;n : int32 ) : int32;
    FUNCTION addchstr ( st : pchar ) : int32;
    FUNCTION addnstr ( st : pchar;n : int32 ) : int32;
    FUNCTION addstr ( st : pchar ) : int32;
    FUNCTION attroff ( at : int32 ) : int32;
    FUNCTION attron ( at : int32 ) : int32;
    FUNCTION attrset ( at : int32 ) : int32;
    FUNCTION bkgd ( ch : int32 ) : int32;
    PROCEDURE bkgdset ( ch : int32 );
    FUNCTION clear : int32;
    FUNCTION clrtobot : int32;
    FUNCTION clrtoeol : int32;
    FUNCTION delch : int32;
    FUNCTION deleteln : int32;
    FUNCTION echochar ( c : int32 ) : int32;
    FUNCTION erase : int32;
    FUNCTION getch : int32;
    FUNCTION getstr ( st : pchar ) : int32;
    FUNCTION inch : int32;
    FUNCTION inchnstr ( s : pchar;n : int32 ) : int32;
    FUNCTION inchstr ( s : pchar ) : int32;
    FUNCTION innstr ( s : pchar;n : int32 ) : int32;
    FUNCTION insch ( c : int32 ) : int32;
    FUNCTION insdelln ( n : int32 ) : int32;
    FUNCTION insertln : int32;
    FUNCTION insnstr ( s : pchar;n : int32 ) : int32;
    FUNCTION insstr ( s : pchar ) : int32;
    FUNCTION instr ( s : pchar ) : int32;
    FUNCTION move ( y, x : int32 ) : int32;
    FUNCTION refresh : int32;
    FUNCTION scrl ( n : int32 ) : int32;
    FUNCTION setscrreg ( t, b : int32 ) : int32;
    FUNCTION standend : int32;
    FUNCTION standout : int32;
    FUNCTION timeout ( delay : int32 ) : int32;
    FUNCTION wdeleteln ( win : pwindow ) : int32;
    FUNCTION winsertln ( win : pwindow ) : int32;
    {
       mv functions
     }
    FUNCTION mvwaddch ( win : pwindow;y, x : int32; ch : chtype ) : int32;
    FUNCTION mvwaddchnstr ( win : pwindow;y, x : int32;st : pchar;n : int32 ) : int32;
    FUNCTION mvwaddchstr ( win : pwindow;y, x : int32;st : pchar ) : int32;
    FUNCTION mvwaddnstr ( win : pwindow;y, x : int32;st : pchar;n : int32 ) : int32;
    FUNCTION mvwaddstr ( win : pwindow;y, x : int32;st : pchar ) : int32;
    FUNCTION mvwdelch ( win : pwindow;y, x : int32 ) : int32;
    FUNCTION mvwgetch ( win : pwindow;y, x : int32 ) : int32;
    FUNCTION mvwgetnstr ( win : pwindow;y, x : int32;st : pchar;n : int32 ) : int32;
    FUNCTION mvwgetstr ( win : pwindow;y, x : int32;st : pchar ) : int32;
    FUNCTION mvwhline ( win : pwindow;y, x : int32;c : chtype;n : int32 ) : int32;
    FUNCTION mvwinch ( win : pwindow;y, x : int32 ) : int32;
    FUNCTION mvwinchnstr ( win : pwindow;y, x : int32;s : pchar; n : int32 ) : int32;
    FUNCTION mvwinchstr ( win : pwindow;y, x : int32;s : pchar ) : int32;
    FUNCTION mvwinnstr ( win : pwindow;y, x : int32;s : pchar;n : int32 ) : int32;
    FUNCTION mvwinsch ( win : pwindow;y, x : int32;c : chtype ) : int32;
    FUNCTION mvwinsnstr ( win : pwindow;y, x : int32;s : pchar;n : int32 ) : int32;
    FUNCTION mvwinsstr ( win : pwindow;y, x : int32;s : pchar ) : int32;
    FUNCTION mvwinstr ( win : pwindow;y, x : int32;s : pchar ) : int32;
    FUNCTION mvwvline ( win : pwindow;y, x : int32;c : chtype;n : int32 ) : int32;
    FUNCTION mvaddch ( y, x, ch : int32 ) : int32;
    FUNCTION mvaddchnstr ( y, x : int32; st : pchar;n : int32 ) : int32;
    FUNCTION mvaddchstr ( y, x : int32; st : pchar ) : int32;
    FUNCTION mvaddnstr ( y, x : int32; st : pchar;n : int32 ) : int32;
    FUNCTION mvaddstr ( y, x : int32; st : pchar ) : int32;
    FUNCTION mvdelch ( y, x : int32 ) : int32;
    FUNCTION mvgetch ( y, x : int32 ) : int32;
    FUNCTION mvgetnstr ( y, x : int32; st : pchar;n : int32 ) : int32;
    FUNCTION mvgetstr ( y, x : int32; st : pchar ) : int32;
    FUNCTION mvhline ( y, x : int32;c : chtype;n : int32 ) : int32;
    FUNCTION mvinch ( y, x : int32 ) : int32;
    FUNCTION mvinchnstr ( y, x : int32; s : pchar;n : int32 ) : int32;
    FUNCTION mvinchstr ( y, x : int32; s : pchar ) : int32;
    FUNCTION mvinnstr ( y, x : int32; s : pchar;n : int32 ) : int32;
    FUNCTION mvinsch ( y, x : int32;c : chtype ) : int32;
    FUNCTION mvinsnstr ( y, x : int32; s : pchar;n : int32 ) : int32;
    FUNCTION mvinsstr ( y, x : int32; s : pchar ) : int32;
    FUNCTION mvinstr ( y, x : int32; s : pchar ) : int32;
    FUNCTION mvvline ( y, x, c, n : int32 ) : int32;
    FUNCTION attr_get : int32;
    FUNCTION attr_off ( a : int32 ) : int32;
    FUNCTION attr_on ( a : int32 ) : int32;
    FUNCTION attr_set ( a : int32 ) : int32;
    FUNCTION chgat ( n, a, c, o : int32 ) : int32;
    FUNCTION getbkgd ( win : pwindow ) : int32;
    FUNCTION slk_attr_off ( a : int32 ) : int32;
    FUNCTION slk_attr_on ( a : int32 ) : int32;
    FUNCTION slk_attr_set ( a : int32 ) : int32;
    FUNCTION vid_attr ( a : int32 ) : int32;
    FUNCTION wattr_get ( win : pwindow ) : int32;
    {
       Pseudo-character tokens outside ASCII range.  The curses wgetch() function
       will return any given one of these only if the corresponding k- capability
       is defined in your terminal's terminfo entry.
     }
    CONST                       {octal}
       KEY_CODE_YES = 256;      {0400}
       KEY_MIN = 257;           {0401}
       KEY_BREAK = 257;         {0401}
       KEY_DOWN = 258;          {0402}
       KEY_UP = 259;            {0403}
       KEY_LEFT = 260;          {0404}
       KEY_RIGHT = 261;         {0405}
       KEY_HOME = 262;          {0406}
       KEY_BACKSPACE = 263;     {0407}
       KEY_F0 = 264;            {0410}

    FUNCTION KEY_F ( n : int32 ) : int32;

    CONST
       KEY_DL = 328;            {0510}
       KEY_IL = 329;            {0511}
       KEY_DC = 330;            {0512}
       KEY_IC = 331;            {0513}
       KEY_EIC = 332;           {0514}
       KEY_CLEAR = 333;         {0515}
       KEY_EOS = 334;           {0516}
       KEY_EOL = 335;           {0517}
       KEY_SF = 336;            {0520}
       KEY_SR = 337;            {0521}
       KEY_NPAGE = 338;         {0522}
       KEY_PPAGE = 339;         {0523}
       KEY_STAB = 340;          {0524}
       KEY_CTAB = 341;          {0525}
       KEY_CATAB = 342;         {0526}
       KEY_ENTER = 343;         {0527}
       KEY_SRESET = 344;        {0530}
       KEY_RESET = 345;         {0531}
       KEY_PRINT = 346;         {0532}
       KEY_LL = 347;            {0533}
       KEY_A1 = 348;            {0534}
       KEY_A3 = 349;            {0535}
       KEY_B2 = 350;            {0536}
       KEY_C1 = 351;            {0537}
       KEY_C3 = 352;            {0540}
       KEY_BTAB = 353;          {0541}
       KEY_BEG = 354;           {0542}
       KEY_CANCEL = 355;        {0543}
       KEY_CLOSE = 356;         {0544}
       KEY_COMMAND = 357;       {0545}
       KEY_COPY = 358;          {0546}
       KEY_CREATE = 359;        {0547}
       KEY_END = 360;           {0550}
       KEY_EXIT = 361;          {0551}
       KEY_FIND = 362;          {0552}
       KEY_HELP = 363;          {0553}
       KEY_MARK = 364;          {0554}
       KEY_MESSAGE = 365;       {0555}
       KEY_MOVE = 366;          {0556}
       KEY_NEXT = 367;          {0557}
       KEY_OPEN = 368;          {0560}
       KEY_OPTIONS = 369;       {0561}
       KEY_PREVIOUS = 370;      {0562}
       KEY_REDO = 371;          {0563}
       KEY_REFERENCE = 372;     {0564}
       KEY_REFRESH = 373;       {0565}
       KEY_REPLACE = 374;       {0566}
       KEY_RESTART = 375;       {0567}
       KEY_RESUME = 376;        {0570}
       KEY_SAVE = 377;          {0571}
       KEY_SBEG = 378;          {0572}
       KEY_SCANCEL = 379;       {0573}
       KEY_SCOMMAND = 380;      {0574}
       KEY_SCOPY = 381;         {0575}
       KEY_SCREATE = 382;       {0576}
       KEY_SDC = 383;           {0577}
       KEY_SDL = 384;           {0600}
       KEY_SELECT = 385;        {0601}
       KEY_SEND = 386;          {0602}
       KEY_SEOL = 387;          {0603}
       KEY_SEXIT = 388;         {0604}
       KEY_SFIND = 389;         {0605}
       KEY_SHELP = 390;         {0606}
       KEY_SHOME = 391;         {0607}
       KEY_SIC = 392;           {0610}
       KEY_SLEFT = 393;         {0611}
       KEY_SMESSAGE = 394;      {0612}
       KEY_SMOVE = 395;         {0613}
       KEY_SNEXT = 396;         {0614}
       KEY_SOPTIONS = 397;      {0615}
       KEY_SPREVIOUS = 398;     {0616}
       KEY_SPRINT = 399;        {0617}
       KEY_SREDO = 400;         {0620}
       KEY_SREPLACE = 401;      {0621}
       KEY_SRIGHT = 402;        {0622}
       KEY_SRSUME = 403;        {0623}
       KEY_SSAVE = 404;         {0624}
       KEY_SSUSPEND = 405;      {0625}
       KEY_SUNDO = 406;         {0626}
       KEY_SUSPEND = 407;       {0627}
       KEY_UNDO = 408;          {0630}
       KEY_MOUSE = 409;         {0631}
       KEY_RESIZE = 410;        {0632}
       KEY_MAX = 511;           {0777}

    FUNCTION mcprint ( _para1 : pchar; _para2 : int32 ) : int32;external name 'mcprint';
    FUNCTION has_key ( _para1 : int32 ) : IntBool; external name 'has_key';

IMPLEMENTATION

FUNCTION wgetstr ( w : pwindow;s : pchar ) : int32;
BEGIN
  wgetstr := wgetnstr ( w, s, - ( 1 ) );
END;

FUNCTION getnstr ( s : pchar;n : int32 ) : int32;
BEGIN
  getnstr := wgetnstr ( stdscr, s, n );
END;

FUNCTION setterm ( term : int32 ) : int32;
BEGIN { The setterm routine is being replaced by setupterm }
  {
  setterm:=setupterm(term,1,plongint(0));
  }
  setterm := 0;
END;

FUNCTION fixterm : int32;
BEGIN
  fixterm := reset_prog_mode;
END;

FUNCTION resetterm : int32;
BEGIN
  resetterm := reset_shell_mode;
END;

FUNCTION saveterm : int32;
BEGIN
  saveterm := def_prog_mode;
END;

FUNCTION crmode : int32;
BEGIN
  crmode := cbreak;
END;

FUNCTION nocrmode : int32;
BEGIN
  nocrmode := nocbreak;
END;

PROCEDURE getsyx ( VAR y, x : int32 );
BEGIN
  getyx ( stdscr, y, x );
END;

FUNCTION getattrs ( win : pwindow ) : int32;
VAR
  if_local1 : int32;
BEGIN
  IF win <> NIL THEN
    if_local1 := win^._attrs
  ELSE
    if_local1 := A_NORMAL;
  getattrs := if_local1;
END;

FUNCTION getcurx ( win : pwindow ) : int32;
VAR
  if_local1 : int32;
BEGIN
  IF win <> NIL THEN
    if_local1 := win^._curx
  ELSE
    if_local1 := ERR;
  getcurx := if_local1;
END;

FUNCTION getcury ( win : pwindow ) : int32;
VAR
  if_local1 : int32;
BEGIN
  IF win <> NIL THEN
    if_local1 := win^._cury
  ELSE
    if_local1 := ERR;
  getcury := if_local1;
END;
FUNCTION getbegx ( win : pwindow ) : int32;
VAR
  if_local1 : int32;

BEGIN
  IF win <> NIL THEN
    if_local1 := win^._begx
  ELSE
    if_local1 := ERR;
  getbegx := if_local1;
END;

FUNCTION getbegy ( win : pwindow ) : int32;
VAR
  if_local1 : int32;
BEGIN
  IF win <> NIL THEN
    if_local1 := win^._begy
  ELSE
    if_local1 := ERR;
  getbegy := if_local1;
END;

FUNCTION getmaxx ( win : pwindow ) : int32;
VAR
  if_local1 : int32;
BEGIN
  IF win <> NIL THEN
    if_local1 := ( win^._maxx ) + 1
  ELSE
    if_local1 := ERR;
  getmaxx := if_local1;
END;

FUNCTION getmaxy ( win : pwindow ) : int32;
VAR
  if_local1 : int32;
BEGIN
  IF win <> NIL THEN
    if_local1 := ( win^._maxy ) + 1
  ELSE
    if_local1 := ERR;
  getmaxy := if_local1;
END;

FUNCTION getparx ( win : pwindow ) : int32;
VAR
  if_local1 : int32;
BEGIN
  IF win <> NIL THEN
    if_local1 := win^._parx
  ELSE
    if_local1 := ERR;
  getparx := if_local1;
END;

FUNCTION getpary ( win : pwindow ) : int32;
VAR
  if_local1 : int32;
BEGIN
  IF win <> NIL THEN
    if_local1 := win^._pary
  ELSE
    if_local1 := ERR;
  getpary := if_local1;
END;

FUNCTION wstandout ( win : pwindow ) : int32;
BEGIN
  wstandout := wattr_set ( win, A_STANDOUT );
END;

FUNCTION wstandend ( win : pwindow ) : int32;
BEGIN
  wstandend := wattr_set ( win, A_NORMAL );
END;

(*
function wattron(win : pwindow;at : int32) : int32;
begin
  wattron:=wattr_on(win,at);
end;

function wattroff(win : pwindow;at : int32) : int32;
begin
  wattroff:=wattr_off(win,at);
end;

function wattrset(win : pwindow;at : int32) : int32;
begin
  wattrset:=wattr_set(win,at);
end;
*)
FUNCTION scroll ( win : pwindow ) : int32;
BEGIN
  scroll := wscrl ( win, 1 );
END;

FUNCTION touchwin ( win : pwindow ) : int32;
BEGIN
  touchwin := wtouchln ( win, 0, getmaxy ( win ), 1 );
END;

FUNCTION touchline ( win : pwindow;s, c : int32 ) : int32;
BEGIN
  touchline := wtouchln ( win, s, c, 1 );
END;

FUNCTION untouchwin ( win : pwindow ) : int32;
BEGIN
  untouchwin := wtouchln ( win, 0, getmaxy ( win ), 0 );
END;

FUNCTION box ( win : pwindow;v, h : int32 ) : int32;
BEGIN
  box := wborder ( win, v, v, h, h, 0, 0, 0, 0 );
END;

FUNCTION border ( ls, rs, ts, bs, tl, tr, bl, br : int32 ) : int32;
BEGIN
  border := wborder ( stdscr, ls, rs, ts, bs, tl, tr, bl, br );
END;

FUNCTION hline ( ch, n : int32 ) : int32;
BEGIN
  hline := whline ( stdscr, ch, n );
END;

FUNCTION vline ( ch, n : int32 ) : int32;
BEGIN
  vline := wvline ( stdscr, ch, n );
END;

FUNCTION winstr ( w : pwindow;s : pchar ) : int32;
BEGIN
  winstr := winnstr ( w, s, - ( 1 ) );
END;

FUNCTION winchstr ( w : pwindow;s : pchar ) : int32;
BEGIN
  winchstr := winchnstr ( w, s, - 1 );
END;

FUNCTION winsstr ( w : pwindow;s : pchar ) : int32;
BEGIN
  winsstr := winsnstr ( w, s, - ( 1 ) );
END;

FUNCTION redrawwin ( w : pwindow ) : int32;
BEGIN
  redrawwin := wredrawln ( w, 0, ( w^._maxy ) + 1 );
END;

FUNCTION waddstr ( win : pwindow;st : pchar ) : int32;
BEGIN
  waddstr := waddnstr ( win, st, - 1 );
END;

FUNCTION waddchstr ( win : pwindow;st : pchar ) : int32;
BEGIN
  waddchstr := waddchnstr ( win, st, - 1 );
END;

FUNCTION addch ( ch : int32 ) : int32;
BEGIN
  addch := waddch ( stdscr, ch );
END;

FUNCTION addchnstr ( st : pchar;n : int32 ) : int32;
BEGIN
  addchnstr := waddchnstr ( stdscr, st, n );
END;

FUNCTION addchstr ( st : pchar ) : int32;
BEGIN
  addchstr := waddchstr ( stdscr, st );
END;

FUNCTION addnstr ( st : pchar;n : int32 ) : int32;
BEGIN
  addnstr := waddnstr ( stdscr, st, n );
END;

FUNCTION addstr ( st : pchar ) : int32;
BEGIN
  addstr := waddnstr ( stdscr, st, - 1 );
END;

FUNCTION attroff ( at : int32 ) : int32;
BEGIN
  attroff := wattroff ( stdscr, at );
END;

FUNCTION attron ( at : int32 ) : int32;
BEGIN
  attron := wattron ( stdscr, at );
END;

FUNCTION attrset ( at : int32 ) : int32;
BEGIN
  attrset := wattrset ( stdscr, at );
END;

FUNCTION bkgd ( ch : int32 ) : int32;
BEGIN
  bkgd := wbkgd ( stdscr, ch );
END;

PROCEDURE bkgdset ( ch : int32 );
BEGIN
  wbkgdset ( stdscr, ch );
END;

FUNCTION clear : int32;
BEGIN
  clear := wclear ( stdscr );
END;

FUNCTION clrtobot : int32;
BEGIN
  clrtobot := wclrtobot ( stdscr );
END;

FUNCTION clrtoeol : int32;
BEGIN
  clrtoeol := wclrtoeol ( stdscr );
END;

FUNCTION delch : int32;
BEGIN
  delch := wdelch ( stdscr );
END;

FUNCTION deleteln : int32;
BEGIN
  deleteln := winsdelln ( stdscr, - 1 );
END;

FUNCTION echochar ( c : int32 ) : int32;
BEGIN
  echochar := wechochar ( stdscr, c );
END;

FUNCTION erase : int32;
BEGIN
  erase := werase ( stdscr );
END;

FUNCTION getch : int32;
BEGIN
  getch := wgetch ( stdscr );
END;

FUNCTION getstr ( st : pchar ) : int32;
BEGIN
  getstr := wgetstr ( stdscr, st );
END;

FUNCTION inch : int32;
BEGIN
  inch := winch ( stdscr );
END;

FUNCTION inchnstr ( s : pchar;n : int32 ) : int32;
BEGIN
  inchnstr := winchnstr ( stdscr, s, n );
END;

FUNCTION inchstr ( s : pchar ) : int32;
BEGIN
  inchstr := winchstr ( stdscr, s );
END;

FUNCTION innstr ( s : pchar;n : int32 ) : int32;
BEGIN
  innstr := winnstr ( stdscr, s, n );
END;

FUNCTION insch ( c : int32 ) : int32;
BEGIN
  insch := winsch ( stdscr, c );
END;

FUNCTION insdelln ( n : int32 ) : int32;
BEGIN
  insdelln := winsdelln ( stdscr, n );
END;

FUNCTION insertln : int32;
BEGIN
  insertln := winsdelln ( stdscr, 1 );
END;

FUNCTION insnstr ( s : pchar;n : int32 ) : int32;
BEGIN
  insnstr := winsnstr ( stdscr, s, n );
END;

FUNCTION insstr ( s : pchar ) : int32;
BEGIN
  insstr := winsstr ( stdscr, s );
END;

FUNCTION instr ( s : pchar ) : int32;
BEGIN
  instr := winstr ( stdscr, s );
END;

FUNCTION move ( y, x : int32 ) : int32;
BEGIN
  move := wmove ( stdscr, y, x );
END;

FUNCTION refresh : int32;
BEGIN
  refresh := wrefresh ( stdscr );
END;

FUNCTION scrl ( n : int32 ) : int32;
BEGIN
  scrl := wscrl ( stdscr, n );
END;

FUNCTION setscrreg ( t, b : int32 ) : int32;
BEGIN
  setscrreg := wsetscrreg ( stdscr, t, b );
END;

FUNCTION standend : int32;
BEGIN
  standend := wstandend ( stdscr );
END;

FUNCTION standout : int32;
BEGIN
  standout := wstandout ( stdscr );
END;

FUNCTION timeout ( delay : int32 ) : int32;
BEGIN
  timeout := wtimeout ( stdscr, delay );
END;

FUNCTION wdeleteln ( win : pwindow ) : int32;
BEGIN
  wdeleteln := winsdelln ( win, - 1 );
END;

FUNCTION winsertln ( win : pwindow ) : int32;
BEGIN
  winsertln := winsdelln ( win, 1 );
END;

FUNCTION mvaddch ( y, x, ch : int32 ) : int32;
BEGIN
  mvaddch := mvwaddch ( stdscr, y, x, ch );
END;

FUNCTION mvaddchnstr ( y, x : int32;st : pchar;n : int32 ) : int32;
BEGIN
  mvaddchnstr := mvwaddchnstr ( stdscr, y, x, st, n );
END;

FUNCTION mvaddchstr ( y, x : int32;st : pchar ) : int32;
BEGIN
  mvaddchstr := mvwaddchstr ( stdscr, y, x, st );
END;

FUNCTION mvaddnstr ( y, x : int32;st : pchar; n : int32 ) : int32;
BEGIN
  mvaddnstr := mvwaddnstr ( stdscr, y, x, st, n );
END;

FUNCTION mvaddstr ( y, x : int32;st : pchar ) : int32;
BEGIN
  mvaddstr := mvwaddstr ( stdscr, y, x, st );
END;

FUNCTION mvdelch ( y, x : int32 ) : int32;
BEGIN
  mvdelch := mvwdelch ( stdscr, y, x );
END;

FUNCTION mvgetch ( y, x : int32 ) : int32;
BEGIN
  mvgetch := mvwgetch ( stdscr, y, x );
END;

FUNCTION mvgetnstr ( y, x : int32;st : pchar;n : int32 ) : int32;
BEGIN
  mvgetnstr := mvwgetnstr ( stdscr, y, x, st, n );
END;

FUNCTION mvgetstr ( y, x : int32;st : pchar ) : int32;
BEGIN
  mvgetstr := mvwgetstr ( stdscr, y, x, st );
END;

FUNCTION mvhline ( y, x : int32;c : chtype;n : int32 ) : int32;
BEGIN
  mvhline := mvwhline ( stdscr, y, x, c, n );
END;

FUNCTION mvinch ( y, x : int32 ) : int32;
BEGIN
  mvinch := mvwinch ( stdscr, y, x );
END;

FUNCTION mvinchnstr ( y, x : int32;s : pchar;n : int32 ) : int32;
BEGIN
  mvinchnstr := mvwinchnstr ( stdscr, y, x, s, n );
END;

FUNCTION mvinchstr ( y, x : int32;s : pchar ) : int32;
BEGIN
  mvinchstr := mvwinchstr ( stdscr, y, x, s );
END;

FUNCTION mvinnstr ( y, x : int32;s : pchar;n : int32 ) : int32;
BEGIN
  mvinnstr := mvwinnstr ( stdscr, y, x, s, n );
END;

FUNCTION mvinsch ( y, x : int32;c : chtype ) : int32;
BEGIN
  mvinsch := mvwinsch ( stdscr, y, x, c );
END;

FUNCTION mvinsnstr ( y, x : int32;s : pchar;n : int32 ) : int32;
BEGIN
  mvinsnstr := mvwinsnstr ( stdscr, y, x, s, n );
END;

FUNCTION mvinsstr ( y, x : int32;s : pchar ) : int32;
BEGIN
  mvinsstr := mvwinsstr ( stdscr, y, x, s );
END;

FUNCTION mvinstr ( y, x : int32;s : pchar ) : int32;
BEGIN
  mvinstr := mvwinstr ( stdscr, y, x, s );
END;

FUNCTION mvvline ( y, x, c, n : int32 ) : int32;
BEGIN
  mvvline := mvwvline ( stdscr, y, x, c, n );
END;

FUNCTION attr_get : int32;
BEGIN
  attr_get := wattr_get ( stdscr );
END;

FUNCTION attr_off ( a : int32 ) : int32;
BEGIN
  attr_off := wattr_off ( stdscr, a );
END;

FUNCTION attr_on ( a : int32 ) : int32;
BEGIN
  attr_on := wattr_on ( stdscr, a );
END;

FUNCTION attr_set ( a : int32 ) : int32;
BEGIN
  attr_set := wattr_set ( stdscr, a );
END;

FUNCTION chgat ( n, a, c, o : int32 ) : int32;
BEGIN
 chgat := wchgat ( stdscr, n, a, c, pointer ( ptrint ( o ) ) );
END;

FUNCTION getbkgd ( win : pwindow ) : int32;
BEGIN
  getbkgd := win^._bkgd;
END;

FUNCTION mvchgat ( y, x, n, a, c, o : int32 ) : int32;
BEGIN
  mvchgat := mvwchgat ( stdscr, y, x, n, a, c, o );
END;

FUNCTION slk_attr_off ( a : int32 ) : int32;
BEGIN
  slk_attr_off := slk_attroff ( a );
END;

FUNCTION slk_attr_on ( a : int32 ) : int32;
BEGIN
  slk_attr_on := slk_attron ( a );
END;

FUNCTION slk_attr_set ( a : int32 ) : int32;
BEGIN
  slk_attr_set := slk_attrset ( a );
END;

FUNCTION vid_attr ( a : int32 ) : int32;
BEGIN
  vid_attr := vidattr ( a );
END;

FUNCTION wattr_get ( win : pwindow ) : int32;
BEGIN
  wattr_get := win^._attrs;
END;

FUNCTION KEY_F ( n : int32 ) : int32;
BEGIN
  KEY_F := KEY_F0 + n;
END;

PROCEDURE getyx ( win : pwindow; VAR y, x : int32 );
BEGIN
  X := ERR;
  Y := ERR;
  IF Win <> NIL THEN
   BEGIN
     Y := win^._cury;
     X := Win^._curx;
   END;
END;

PROCEDURE getbegyx ( win : pwindow; VAR y, x : int32 );
BEGIN
  X := ERR;
  Y := ERR;
  IF Win <> NIL THEN
   BEGIN
     Y := win^._begy;
     X := Win^._begx;
   END;
END;

PROCEDURE getmaxyx ( win : pwindow; VAR y, x : int32 );
BEGIN
  X := ERR;
  Y := ERR;
  IF Win <> NIL THEN
   BEGIN
     Y := win^._maxy + 1;
     X := Win^._maxx + 1;
   END;
END;

PROCEDURE getparyx ( win : pwindow; VAR y, x : int32 );
BEGIN
  X := ERR;
  Y := ERR;
  IF Win <> NIL THEN
   BEGIN
     Y := win^._pary;
     X := Win^._parx;
   END;
END;
(* kjw, 08/23/2000, external in v4.2
function winch (win : PWindow) : int32;
begin
  if win<>nil then
   winch:=win^._line[win^._cury].text[Win^ ._curx]
  else
   winch:=0;
end;

function wattr_set(win : pwindow; at : int32) : int32;
begin
  If win<>nil then
   begin
     win^._attrs := at;
     wattr_set:=at;
   end
  else
   wattr_set:=0;
end;
*)
PROCEDURE setsyx ( y, x : int32 );
BEGIN
  stdscr^._cury := y;
  stdscr^._curx := x;
END;

FUNCTION  mvwaddch ( win : pwindow;y, x : int32; ch : chtype ) : int32;
BEGIN
  IF wmove ( win, y, x ) = ERR THEN
   exit ( ERR )
  ELSE
   exit ( waddch ( win, ch ) )
END;

FUNCTION  mvwaddchnstr ( win : pwindow;y, x : int32;st : pchar;n : int32 ) : int32;
BEGIN
  IF wmove ( win, y, x ) = ERR THEN
   exit ( ERR )
  ELSE
   exit ( waddchnstr ( win, st, n ) )
END;

FUNCTION  mvwaddchstr ( win : pwindow;y, x : int32;st : pchar ) : int32;
BEGIN
  IF wmove ( win, y, x ) = ERR THEN
   exit ( ERR )
  ELSE
   exit ( waddchnstr ( win, st, - 1 ) )
END;

FUNCTION  mvwaddnstr ( win : pwindow;y, x : int32;st : pchar;n : int32 ) : int32;
BEGIN
  IF wmove ( win, y, x ) = ERR THEN
   exit ( ERR )
  ELSE
   exit ( waddnstr ( win, st, n ) )
END;

FUNCTION  mvwaddstr ( win : pwindow;y, x : int32;st : pchar ) : int32;
BEGIN
  IF wmove ( win, y, x ) = ERR THEN
   exit ( ERR )
  ELSE
   exit ( waddnstr ( win, st, - 1 ) )
END;

FUNCTION  mvwdelch ( win : pwindow;y, x : int32 ) : int32;
BEGIN
  IF wmove ( win, y, x ) = ERR THEN
   exit ( ERR )
  ELSE
   exit ( wdelch ( win ) )
END;

FUNCTION  mvwgetch ( win : pwindow;y, x : int32 ) : int32;
BEGIN
  IF wmove ( win, y, x ) = ERR THEN
   exit ( ERR )
  ELSE
   exit ( wgetch ( win ) )
END;

FUNCTION  mvwgetnstr ( win : pwindow;y, x : int32;st : pchar;n : int32 ) : int32;
BEGIN
  IF wmove ( win, y, x ) = ERR THEN
   exit ( ERR )
  ELSE
   exit ( wgetnstr ( win, st, n ) )
END;

FUNCTION  mvwgetstr ( win : pwindow;y, x : int32;st : pchar ) : int32;
BEGIN
  IF wmove ( win, y, x ) = ERR THEN
   exit ( ERR )
  ELSE
   exit ( wgetstr ( win, st ) )
END;

FUNCTION  mvwhline ( win : pwindow;y, x : int32;c : chtype;n : int32 ) : int32;
BEGIN
  IF wmove ( win, y, x ) = ERR THEN
   exit ( ERR )
  ELSE
   exit ( whline ( win, c, n ) )
END;

FUNCTION  mvwinch ( win : pwindow;y, x : int32 ) : int32;
BEGIN
  IF wmove ( win, y, x ) = ERR THEN
   exit ( ERR )
  ELSE
   exit ( winch ( win ) )
END;

FUNCTION  mvwinchnstr ( win : pwindow;y, x : int32;s : pchar; n : int32 ) : int32;
BEGIN
  IF wmove ( win, y, x ) = ERR THEN
   exit ( ERR )
  ELSE
   exit ( winchnstr ( win, s, n ) )
END;

FUNCTION  mvwinchstr ( win : pwindow;y, x : int32;s : pchar ) : int32;
BEGIN
  IF wmove ( win, y, x ) = ERR THEN
   exit ( ERR )
  ELSE
   exit ( winchstr ( win, s ) )
END;

FUNCTION  mvwinnstr ( win : pwindow;y, x : int32;s : pchar;n : int32 ) : int32;
BEGIN
  IF wmove ( win, y, x ) = ERR THEN
   exit ( ERR )
  ELSE
   exit ( winnstr ( win, s, n ) )
END;

FUNCTION  mvwinsch ( win : pwindow;y, x : int32;c : chtype ) : int32;
BEGIN
  IF wmove ( win, y, x ) = ERR THEN
   exit ( ERR )
  ELSE
   exit ( winsch ( win, c ) )
END;

FUNCTION  mvwinsnstr ( win : pwindow;y, x : int32;s : pchar;n : int32 ) : int32;
BEGIN
  IF wmove ( win, y, x ) = ERR THEN
   exit ( ERR )
  ELSE
   exit ( winsnstr ( win, s, n ) )
END;

FUNCTION  mvwinsstr ( win : pwindow;y, x : int32;s : pchar ) : int32;
BEGIN
  IF wmove ( win, y, x ) = ERR THEN
   exit ( ERR )
  ELSE
   exit ( winsstr ( win, s ) )
END;

FUNCTION  mvwinstr ( win : pwindow;y, x : int32;s : pchar ) : int32;
BEGIN
  IF wmove ( win, y, x ) = ERR THEN
   exit ( ERR )
  ELSE
   exit ( winstr ( win, s ) )
END;

FUNCTION  mvwvline ( win : pwindow;y, x : int32;c : chtype;n : int32 ) : int32;
BEGIN
  IF wmove ( win, y, x ) = ERR THEN
   exit ( ERR )
  ELSE
   exit ( wvline ( win, c, n ) )
END;

FUNCTION color_pair ( n : int32 ) : int32;
BEGIN
  color_pair := n SHL 8;
END;

FUNCTION ACS_ULCORNER : chtype;
BEGIN
  ACS_ULCORNER := acs_map ['l'];
END;

FUNCTION ACS_LLCORNER : chtype;
BEGIN
  ACS_LLCORNER := acs_map ['m'];
END;

FUNCTION ACS_URCORNER : chtype;
BEGIN
  ACS_URCORNER := acs_map ['k'];
END;

FUNCTION ACS_LRCORNER : chtype;
BEGIN
  ACS_LRCORNER := acs_map ['j'];
END;

FUNCTION ACS_LTEE : chtype;
BEGIN
  ACS_LTEE := acs_map ['t'];
END;

FUNCTION ACS_RTEE : chtype;
BEGIN
  ACS_RTEE := acs_map ['u'];
END;

FUNCTION ACS_BTEE : chtype;
BEGIN
  ACS_BTEE := acs_map ['v'];
END;

FUNCTION ACS_TTEE : chtype;
BEGIN
  ACS_TTEE := acs_map ['w'];
END;

FUNCTION ACS_HLINE : chtype;
BEGIN
  ACS_HLINE := acs_map ['q'];
END;

FUNCTION ACS_VLINE : chtype;
BEGIN
  ACS_VLINE := acs_map ['x'];
END;

FUNCTION ACS_PLUS : chtype;
BEGIN
  ACS_PLUS := acs_map ['n'];
END;

FUNCTION ACS_S1      : chtype;
BEGIN
  ACS_S1    := acs_map ['o'];
END;

FUNCTION ACS_S9      : chtype;
BEGIN
  ACS_S9    := acs_map ['s'];
END;

FUNCTION ACS_DIAMOND : chtype;
BEGIN
  ACS_DIAMOND := acs_map ['`'];
END;

FUNCTION ACS_CKBOARD : chtype;
BEGIN
  ACS_CKBOARD := acs_map ['a'];
END;

FUNCTION ACS_DEGREE : chtype;
BEGIN
  ACS_DEGREE := acs_map ['f'];
END;

FUNCTION ACS_PLMINUS : chtype;
BEGIN
  ACS_PLMINUS := acs_map ['g'];
END;

FUNCTION ACS_BULLET : chtype;
BEGIN
  ACS_BULLET := acs_map ['~'];
END;

FUNCTION ACS_LARROW : chtype;
BEGIN
  ACS_LARROW := acs_map [','];
END;

FUNCTION ACS_RARROW : chtype;
BEGIN
  ACS_RARROW := acs_map ['+'];
END;

FUNCTION ACS_DARROW : chtype;
BEGIN
  ACS_DARROW := acs_map ['.'];
END;

FUNCTION ACS_UARROW : chtype;
BEGIN
  ACS_UARROW := acs_map ['-'];
END;

FUNCTION ACS_BOARD : chtype;
BEGIN
  ACS_BOARD := acs_map ['h'];
END;

FUNCTION ACS_LANTERN : chtype;
BEGIN
  ACS_LANTERN := acs_map ['i'];
END;

FUNCTION ACS_BLOCK : chtype;
BEGIN
  ACS_BLOCK := acs_map ['0'];
END;

FUNCTION ACS_S3      : chtype;
BEGIN
  ACS_S3    := acs_map ['p'];
END;

FUNCTION ACS_S7      : chtype;
BEGIN
  ACS_S7    := acs_map ['r'];
END;

FUNCTION ACS_LEQUAL : chtype;
BEGIN
  ACS_LEQUAL := acs_map ['y'];
END;

FUNCTION ACS_GEQUAL : chtype;
BEGIN
  ACS_GEQUAL := acs_map ['z'];
END;

FUNCTION ACS_PI      : chtype;
BEGIN
  ACS_PI    := acs_map ['{'];
END;

FUNCTION ACS_NEQUAL : chtype;
BEGIN
  ACS_NEQUAL := acs_map ['|'];
END;

FUNCTION ACS_STERLING : chtype;
BEGIN
  ACS_STERLING := acs_map ['}'];
END;

// BEGIN writeln ( ACS_VLINE );
END.
{
  $Log: ncurses.pp,v $
  Revision 1.9  2005/02/14 17:13:21  peter
    * truncate log

}
